/* convolfft_c subroutine
 * FFT convolution computation
 * IRCOM GROUP - Author : A.Layec
 *
 * Copyright (C) 2007 Alan Layec
 * Copyright Scilab (c)INRIA-ENPC
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

#include "machine.h"
extern  integer C2F(fft842)();

/* convolfft_c routine de calcul de filtre  rponse impulsionnelle finie
 * par la mthode de convolution par fft
 *
 * m1   : taille des vecteurs
 * z1_r : adresse de dpart de la partie relle du vecteur temporel 1 d'entre
 * z1_i : adresse de dpart de la partie imaginaire du vecteur temporel 1 d'entre
 * z2_r : adresse de dpart de la partie relle du vecteur temporel 2 d'entre
 * z2_i : adresse de dpart de la partie imaginaire du vecteur temporel 2 d'entre
 * y_r  : adresse de dpart de la partie relle du vecteur temporel rsultat
 * y_i  : adresse de dpart de la partie imaginaire du vecteur temporel rsultat
 *
 * utilise : fft842 (signal)
 *           complx_c
 *
 * rmq m1 doit-tre de taille en puissance de 2
 */

void convolfft_c(int *m1,double *z1_r,double *z1_i,double *z2_r,double *z2_i,double *y_r,double *y_i)
{
 /*dclaration*/
 int k,ierr;

 /*appel fft*/
 C2F(fft842)((k=0,&k),m1,&z1_r[0],&z1_i[0],&ierr);

 /*appel fft*/
 C2F(fft842)((k=0,&k),m1,&z2_r[0],&z2_i[0],&ierr);

 /*Ralise la multiplication vectorielle complexe*/
 cmplxm_c(m1,&z1_r[0],&z1_i[0],&z2_r[0],&z2_i[0],&y_r[0],&y_i[0]);

 /*appel fft-1*/
 C2F(fft842)((k=1,&k),m1,&y_r[0],&y_i[0],&ierr);

 return;
}
