/* cpft_c subroutine
 * Ideal discrete D Flip-Flop
 * Phase/Frequency Detector 
 * with delay on RAZ state
 * IRCOM GROUP - Author : A.Layec
 *
 * Copyright (C) 2007 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"

/* cpft_c routine de calcul d'un comparateur phase frquence trois tats idal
 * Entres :
 * n    : taille des vecteurs
 * nev  : entre d'activation des horloges des bascules
 *       1: bascule 1
 *       2: bascule 2
 *       3: bascule 1&2 (rmq : non programm)
 *       4: entre RAZ des deux bascules
 * flag  : Tache  effectuer
 *       1: calcul des etats de sorties
 *       3: calcul de la date de remise  zro
 * delay : retard de la remise  zro
 * Sorties :
 * y1    : etat de sortie bascule 1
 * y2    : etat de sortie bascule 2
 * evout : date de la remise  zro
 * Entres/sorties :
 * z1 : tat de sortie prcdent bascule 1
 * z2 : etat de sortie prcdent bascule 2
 * sh : etat du comparateur
 *     0 : activ
 *     1 : desactiv
 */

void cpft_c(int *n,int *nev,int *flag,double *delay,double *z1,double *z2,double *evout,double *y1,double *y2,double *sh)
{
 /*dclaration*/
 int i;
 
 for (i=0;i<(*n);i++)
 {
  if(flag[i]==1)
  {
   switch (nev[i])
   {
    case 1 :
    {
     if(sh[i]!=1)
     {
      if ((z1[i]==0) && (z2[i]==0)) {y1[i] = 1; y2[i] = 0;}
      else if (z1[i] == 1) {y1[i] = 1; y2[i] = 0;}
      else {y1[i] = 1; y2[i] = 0;}
     }
     break;
    }
    case 2 :
    {
     if(sh[i]!=1)
     {
      if ((z1[i] == 0) && (z2[i] == 0)) {y1[i] = 0; y2[i] = 1;}
      else if (z2[i] == 1) {y1[i] = 0; y2[i] = 1;}
      else {y1[i] = 0; y2[i] = 1;}
     }
     break;
    }
    case 4 :
    {
     y1[i]=0;
     y2[i]=0;
     sh[i]=0;
     break;
    }
    break;
   }
  }
  else if(flag[i]==3)
  {
   switch (nev[i])
   {
    case 1 :
    {
     if(z2[i] == 1) {evout[i]=delay[i];}
     sh[i]=1;
     break;
    }
    case 2 :
    {
     if(z1[i] == 1) {evout[i]=delay[i];}
     sh[i]=1;
     break;
    }
    break;
   }
  }
  /*Met en mmoire les tats*/ 
  z1[i]=y1[i];
  z2[i]=y2[i];
 }
 return;
}
