/* modpsk_c subroutine
 * Phase Shift Keying Modulator
 * IRCOM GROUP - Author : A.Layec
 *
 * Copyright (C) 2007 Alan Layec
 *
 * This file is part of modnumlib.
 *
 * modnumlib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * modnumlib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with modnumlib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <math.h>

#include "modnum_lib.h"

/* modpsk_c routine de calcul des composantes I et Q en fonction
 * d'un numro symbole cod par tats de phase (psk)
 *
 * Entres :
 *  n   : dimension 1 des matrices d'entre/sorties (scalaire)
 *  m   : dimension 2 des matrices d'entre/sorties (scalaire)
 *  ns  : nombre d'tat (scalaire)
 *  u   : numro symbole (pointeur sur double d'entre)
 *
 * Sorties :
 * i_c : valeur de la composante i (pointeur sur double de sortie 1)
 * q_c : valeur de la composante q (pointeur sur double de sortie 2)
 *
 * Dpendances :
 * math.h
 */

void modpsk_c(int *n,int *m,int *ns,double *u,double *i_c, double *q_c)
{
 /*Dclaration des variables compteurs*/
 int k,i,l;

 for(l=0;l<(*m);l++) {
   for(i=0;i<(*n);i++) {
     /*rcupration de la valeur du port d'entre*/
     k = (int) u[(*n)*l+i];
     /*Calcul des composantes i et q*/
     i_c[(*n)*l+i] =  cos(M_PI*(2*k+1)/(*ns));
     q_c[(*n)*l+i] = -sin(M_PI*(2*k+1)/(*ns));
   }
 }
 return;
}
