/* modgen Scicos Generic symbol modulator block
 * Type 4 simulation function ver 1.0 - scilab-4.1
 * 25 mars 2007 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "modnum_lib.h"
#include "scicos_block.h"

/* Cette fonction de simulation ralise un
 * modulateur symbole gnrique.
 *
 * Les valeurs I/Q sont donnes en paramtres.
 * A chaque entre de valeurs 0  m-1, une
 * valeur symbole est associe, avec m
 * le nombre d'tats.
 *
 * entres rgulires
 *    u[0..insz[0]-1]  : vecteur des numros symboles
 *
 * sorties rgulires
 *    y1[0..insz[0]-1] : vecteur des composantes I
 *    y2[0..insz[0]-1] : vecteur des composantes Q
 *
 * entre vnementielle : instants de dclenchement.
 * sortie vnementielle : nant
 *
 * paramtres entiers
 *    ipar[0] : m nombre d'tats
 *
 * paramtres rels
 *    rpar[0..ipar[0]-1] :
 *       vecteur des parties relles des valeurs symbole
 *    rpar[ipar[0]..2*ipar[0]-1] :
 *      vecteur des parties imaginaires des valeurs symbole
 */

/*prototype*/
void modgen(scicos_block *block,int flag)
{
 /*Dclaration des variables*/
 double *y1,*y2;
 double *u;
 int nu; /*taille du vecteur d'entre*/
 double *u_r,*u_i;
 int m; /*nombre d'tats*/
 int i;

 /*Rcupration des adresses des ports rguliers*/
 u=(double *)block->inptr[0];
 y1=(double *)block->outptr[0];
 y2=(double *)block->outptr[1];

 /*Rcupration de la taille du vecteur d'entre*/
 nu=block->insz[0];

 /*Rcupration du nombre d'tats*/
 m=block->ipar[0];

 /*Rcupration des valeurs relles et imaginaires du symbole*/
 u_r=(double *)&block->rpar[0];
 u_i=(double *)&block->rpar[m];

 /*
  * Le flag 1 calcule la valeur de I et de Q en fonction
  * du numro symbole u[] et du nombre d'tat m
 */
 if(flag==1||flag==6) {
  /*pour chaque lments du vecteur d'entre*/
  for (i=0;i<nu;i++) {
    /*vrification de la valeur de l'entre*/
    if ((u[i]<0)||(u[i]>(double)(m-1))) {
      set_block_error(-1);
      return;
    }
    else {
      /*Appel routine modgen*/
      modgen_c(&nu,&m,&u[0],&u_r[0],&u_i[0],&y1[0],&y2[0]);
    }
  }
 }
}
