/* dac Scicos Digital to Analog Converter block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 8 Mars 2006 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "scicos_block.h"

/* Cette fonction de calcul ralise la convertion numrique analogique.
 *
 *
 * entre rgulire : u[0..nu-1] vecteur des entres rgulires  convertir
 * sortie rgulire : y[0..ny-1] vecteur des sorties converties
 *
 * entre vnementielle : Dates de dclenchement
 * sortie vnementeille : nant
 *
 * paramtre entier : ipar[0..nu-1]     : Nombre de bits de la valeur entire de sortie
 *                    ipar[nu..2*nu-1]  : flag code complment  2
 *                                        0 : nombre entier non sign
 *                                        1 : nombre entier sign
 *
 * paramtre rel : rpar[0..nu-1]      : pas (ou quantum)
 *                  rpar[nu..2*nu-1]   : valeur minimale ou dbute la conversion
 */

/*
 * dacv_c routine de calcul d'un convertisseur numrique analogique
 *
 * Entres :
 * n       : taille des vecteurs
 * nbit    : longueur en bit de la valeur dcimale de sortie (vecteur)
 * cc2     : flag code complment  2 (vecteur)
 *           0 : nombre entier non sign
 *           1 : nombre entier sign
 * q       : pas de quantification (vecteur)
 * vmin    : valeur minimale du dbut de la quantification (vecteur)
 * u       : vecteur d'entre (vecteur)
 *
 * Sorties :
 * y       : vecteur de sortie du convertisseur (vecteur)
 *
 */
void dacv_c(int *n,int *nbit, int *cc2,double *q,double *vmin,\
            double *u,double *y)
{
 /*Dclaration des variables compteurs*/
 int i;

 /*pour chaque elements*/
 for(i=0;i<(*n);i++)
 {
  /*Realise test sur code complment  2*/
  if(cc2[i]==1)
    /*Calcul sortie analogique pour un code complment  2*/
    y[i]=q[i]*(u[i]+((double)((1<<(nbit[i]-1))-1)))+vmin[i];
  else
    /*Calcul sortie analogique pour un entier non sign*/
    y[i]=q[i]*u[i]+vmin[i];
 }
 return;
}

/*prototype*/
void dac(scicos_block *block,int flag)
{
  /*Dclaration des variables*/
  double *u,*y;

  int nu;

  int nbit;
  int cc2;

  double q;
  double vmin;

  /*Rcupration des adresses des ports rguliers*/
  u=(double *)block->inptr[0];
  y=(double *)block->outptr[0];

  /*Rcupration de la taille du port d'entre rgulier*/
  nu=block->insz[0];

  /*Rcupre le nombre de bit*/
  nbit=block->ipar[0];
  /*Rcupre le type de code*/
  cc2=block->ipar[nu];

  /*Rcupre le quantum*/
  q=block->rpar[0];
  /*Rcupre la grandeur minimale d'entre*/
  vmin=block->rpar[nu];

  /*Appel routine dacv_c*/
  dacv_c(&nu,&nbit,&cc2,&q,&vmin,&u[0],&y[0]);
}
