/* gencw Scicos Constant Wave Generator block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 7 janvier 2005 - IRCOM GROUP - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <stdio.h>
#include <math.h>

#include "modnum_lib.h"
#include "scicos_block.h"

#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif

/* Cette fonction dlivre une onde  frquence constante. 
 *
 * entres rgulires : nant
 * sorties rgulires : sortie du gnrateur
 * entre vnementielle : dtermine la cadence de gnration
 *
 * Paramtres rels   : rpar[0] : amplitude
 *                      rpar[1] : longueur de la priode (en chantillons)
 *                      rpar[2] : variance source de bruit
 *
 * Paramtres entiers : ipar[0] : option (1:cos,2:sin,3:sin/cos)
 *                      ipar[1] : (0: pas de bruit, 1:bruit sur le 2*PI)
 * Etat discret       : valeur de l'angle instantan
 *
 * Rmq : pour un fonctionnement en squentiel ( pas constant) rpar[1] doit tre fix  
 *       To/Te ou To est la priode dsire et Te la priode d'chantillonnage
 */

/*prototype*/
void gencw(scicos_block *block,int flag)
{
 /*dclaration des variables*/
 int i,j;
 int ny;
 int opt;
 double inc; /*incrment de l'angle*/
 double ampl;
 double *y1;
 double *y2=NULL;

 /*rcupration de l'adresse de sortie*/
 y1=(double *)block->outptr[0];

 /*rcupration de la taille de sortie*/
 ny=block->outsz[0];

 /*rcupration des paramtres*/
 ampl=block->rpar[0];
 inc=(2*M_PI)/block->rpar[1];
 opt=block->ipar[0];

 /*test si bruit*/ 
 if (block->ipar[1]==1)
 {
  double n;    /*echantillon bruit*/
  double sig;  /*variance bruit*/
  double mean; /*moyenne bruit*/

  sig=block->rpar[2];
  mean=2.0*M_PI;
  /*Appel noiseblk*/
  noiseblk_c((i=1,&i),(j=1,&j),&sig,&mean,&n);

  inc=(n)/block->rpar[1];
 }

 /*seulement sur flag 1*/
 if(flag==1)
 {
  switch (opt)
  {
   case 1 :
   {
    /*Appel cw_r*/
    cw_c(&ny,&ampl,&opt,&inc,&y1[0],y2,&block->z[0]);
    break;
   }
   case 2 :
   {
    /*Appel cw_r*/
    cw_c(&ny,&ampl,&opt,&inc,y2,&y1[0],&block->z[0]);
    break;
   }
   case 3 :
   {
    /*rcupration de l'adresse de sortie*/
    y2=(double *)block->outptr[1];
    /*Appel cw_r*/
    cw_c(&ny,&ampl,&opt,&inc,&y1[0],&y2[0],&block->z[0]);
    break;
   }
  }

 }
}
