/* binreg Scicos binary right shift register
 * Type 4 simulation function ver 1 - scilab-3.0
 * 14 Mars 2006 - INRIA - Author : A.Layec
 * 20 Mars 2007 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include "scicos_block.h"

/* Cette fonction ralise un registre  dcalage binaire.
 * A chaque vnement prsent sur port vnementiel numro 1
 * la fonction ralise une acquistion de la valeur prsente
 * sur son port d'entre rgulier suppos tre de taille
 * nbit*npack puis dlivre un paquet de taille nbit sur
 * le port de sortie rgulier lorsque un vnement se prsente
 * sur son port vnementiel 2.
 * A chaque vnement 2, un ET logique de nbit bits est ralis
 * et le registre est dcal de nbit bits vers la droite.
 *
 * Entre rgulire :
 *     - [0..nu-1] vecteur des entiers cods sur nbit*npack
 *
 * Sortie rgulire :
 *     - y[0..nu-1] vecteur des entiers cods sur nbit
 *
 * Entre vnementielle :
 *     - nev=1 Instant d'acquisition du mot en entre de
 *             taille nbit*npack
 *     - nev=2 Instant de dlivrance du mot en sortie de
 *             taille Nbit
 *
 * Sortie vnementielle : nant
 *
 * Paramtres entiers :
 *     - ipar[0..nu-1] flag code complment  2 en entre
 *                         0 : nombre entier non sign
 *                         1 : nombre entier sign
 *     - ipar[nu..2*nu-1] flag code complment  2 en sortie
 *                         0 : nombre entier non sign
 *                         1 : nombre entier sign
 *     - ipar[2*nu..3*nu-1] longueur des paquets du vecteur
 *                            d'entre (nbit)
 *     - ipar[3*nu..4*nu-1] nombre de paquet  stocker (npack)
 *
 * Etats discrets :
 *      - z[0..nu-1] valeur paquet
 */

/*prototype*/
void binreg(scicos_block *block,int flag)
{
 double *y;
 double *u;
 int nu;

 double *z;

 int nev;

 int *cc2_in;
 int *cc2_out;
 int *nbit;
 int *npack;

 int i,j,k;

 u = (double *)block->inptr[0];
 y = (double *)block->outptr[0];

 nu = block->insz[0];

 z = &block->z[0];

 nev = block->nevprt;

 cc2_in  = &block->ipar[0];
 cc2_out = &block->ipar[nu];
 nbit    = &block->ipar[2*nu];
 npack   = &block->ipar[3*nu];

 /* compute output */
 /* Le flag 1 rcupre le paquet  dlivrer
  * et dlivre sa valeur dans y[]
  * lorsque le port d'activation est 2 ou 3
  */
 if(flag==1 && (nev==2||nev==3)) {
  for(k=0;k<nu;k++) {
   i = (int)z[k];

   j = i & ((1<<nbit[k])-1);

   if (cc2_out[k]==1) {
    j -= 2<<(nbit[k]-2);
    j &= (2<<(nbit[k]-1)) - 1;
    j -= 2<<(nbit[k]-2);
   }

   y[k] = (double)j;

  }
 }

 /* compute discrete state */
 /* Le flag 2 stocke la valeur en entre dans z[]
  * et la converti en nombre entier non sign lorsque
  * le numro du port d'activation est 1 ou 3
  */
 else if(flag==2 && (nev==1||nev==3)) {
  for(k=0;k<nu;k++) {

   j = (int)u[k];

   if (cc2_in[k]==1) {
     j &= (2<<((nbit[k]*npack[k])-1)) - 1;
   }

   z[k] = (double)j;
  }
 }

 /* Le flag 2 dcale le paquet de nbit vers
  * la droite lorsque le numro du port
  * d'activation est 1 ou 3
  */
 else if((flag==2) && (nev==2)) {
  for(k=0;k<nu;k++) {
   i = (int)z[k];

   i = i>>nbit[k];

   z[k] = (double)i;
  }
 }
}
